#include "COMMON.CH"

#DEFINE BASE_DATE    "1900"

*!*********************************************************************
*!
*!       Function: TD2JUL()
*!
*!     Parameters: Time and Date
*!                 Time in TIME() format
*!                 Date in DATE() format
*!
*!        Returns: A julian number
*!
*!*********************************************************************
function TD2JUL( CTIME, DDATE )

return DDATE - ctod( '01/01/' + BASE_DATE ) + ;
              ( PRB_INT( TTOS( CTIME ) / 100000,, 5 ) )

*!*********************************************************************
*!
*!       Function: JUL2TIME()
*!
*!     Parameters: A Julian Number
*!
*!        Returns: Time
*!
*!*********************************************************************
function JUL2TIME( JUL1 )

LOCAL TOTSECS, H, M, S

TOTSECS := val( right( str( JUL1, 14, 5 ), 5 ) )

H       := PRB_INT( TOTSECS / 3600 )
TOTSECS := TOTSECS % 3600
M       := PRB_INT( TOTSECS / 60 )
S       := TOTSECS % 60

return STRZERO( H, 2 ) + ':' + ;
       STRZERO( M, 2 ) + ':' + ;
       STRZERO( S, 2 )

*!*********************************************************************
*!
*!       Function: JUL2DATE()
*!
*!     Parameters: A Julian Number
*!
*!        Returns: Date
*!
*!*********************************************************************
function JUL2DATE( JUL1 )

return ctod('01/01/' + BASE_DATE ) + PRB_INT( JUL1 )

*!*********************************************************************
*!
*!       Function: DIFF2JUL()
*!
*!     Parameters: two julian numbers (in any order)
*!
*!        Returns: The number of Hours, minutes and seconds
*!                 between them in HHHHHHH:MM:SS format
*!
*!*********************************************************************
function DIFF2JUL( JUL1, JUL2 )

LOCAL J1, J2, D, TOTSECS, M, S, H

J1 := min( JUL1, JUL2 )
J2 := max( JUL1, JUL2 )
D  := PRB_INT( J2 ) - PRB_INT( J1 )

TOTSECS := val( right( str( J2, 14, 5 ) , 5 ) ) - ;
           val( right( str( J1, 14, 5 ) , 5 ) )

D       -= PRB_INT( abs( TOTSECS ) / 86400 ) + 1
TOTSECS := PRB_INT( TOTSECS % 86400 ) + 86400

H       := PRB_INT( TOTSECS / 3600 ) + ( D * 24 )
TOTSECS := PRB_INT( TOTSECS % 3600 )
M       := PRB_INT( TOTSECS / 60 )
S       := PRB_INT( TOTSECS % 60 )

if S > 59
  M ++
  S := 0
  if M       > 59
    H ++
    M := 0
  endif
endif

return STRZERO( H, 7 ) + ':' + ;
       STRZERO( M, 2 ) + ':' + ;
       STRZERO( S, 2 )

*!*********************************************************************
*!
*!       Function: JUL_DIFF()
*!
*!     Parameters: A Julian number and some minutes (+/-)
*!
*!        Returns: A new julian number adjusted by that number of
*!                 minutes.
*!
*!*********************************************************************
function JUL_DIFF( JUL1, OFFSET )

LOCAL D, S

D := PRB_INT( JUL1 )

S := val( right( str( JUL1, 14, 5 ), 5 ) ) + ( PRB_INT( OFFSET * 60 ) )

do case
case S < 0
  D -= PRB_INT( abs( S ) / 86400 ) + 1
  S := PRB_INT( abs( S ) % 86400 )
case S > 86399
  D += PRB_INT( S / 86400 )
  S := PRB_INT( S % 86400 )
endcase

return PRB_INT( D + ( S / 100000 ),, 5 )

*!*********************************************************************
*!
*!       Function: JUL_BTW()
*!
*!     Parameters: Three Julian numbers
*!
*!        Returns: .t. or .f. depending on whether UNK is between
*!                 JUL1 AND JUL2  (this is inclusive)
*!
*!*********************************************************************
function JUL_BTW( JUL1, JUL2, UNK )

return ( JUL1 <= UNK ) .and. ( UNK <= JUL2 )

*!*********************************************************************
*!
*!       Function: JUL_BTWX()
*!
*!     Parameters: Three Julian numbers
*!
*!        Returns: .t. or .f. depending on whether UNK is between
*!                 JUL1 AND JUL2  (this is exclusive)
*!
*!*********************************************************************
function JUL_BTWX( JUL1, JUL2, UNK)

return ( JUL1 < UNK ) .and. ( UNK < JUL2 )

*!*********************************************************************
*!
*!       Function: JUL_SEEK()
*!
*!     Parameters: Three Julian Numbers
*!
*!        Returns: A 1 if UNK is closer to to JUL1, otherwise a 2
*!
*!*********************************************************************
function JUL_SEEK( JUL1, JUL2, UNK )

return iif( DIFF2JUL( JUL1, UNK ) < DIFF2JUL( JUL2, UNK ), 1, 2 )

*!*********************************************************************
*!
*!       Function: JUL_AVE()
*!
*!     Parameters: Two julian numbers in any order
*!
*!        Returns: A julian number that is exactly between them.
*!
*!*********************************************************************
function JUL_AVE( JUL1, JUL2 )

LOCAL DIFF, H, M, S

DIFF := DIFF2JUL( min( JUL1, JUL2 ), max( JUL1, JUL2 ) )

H := val(   left( DIFF, 7 ) )
M := val( substr( DIFF, 9, 2 ) )
S := val( substr( DIFF, 12, 2 ) )

return JUL_DIFF( min( JUL1, JUL2 ), ( ( S / 60 ) + M + ( H * 60 ) ) / 2 )

*!*********************************************************************
*!
*!       Function: TTOS()
*!
*!     Parameters: A Time string in HH:MM:SS format
*!
*!        Returns: The number of seconds since midnight
*!
*!*********************************************************************
function TTOS( CTIME )

return ( val( substr( CTIME, 7, 2 ) )        ) + ;
       ( val( substr( CTIME, 4, 2 ) ) * 60   ) + ;
       ( val( substr( CTIME, 1, 2 ) ) * 3600 )

*!*********************************************************************
*!
*!       Function: TTOMS()
*!
*!     Parameters: A Time string in HH:MM:SS format
*!
*!        Returns: The number of minutes since midnight
*!                 with seconds represented as 100THS OF MINUTES
*!
*!*********************************************************************
function TTOMS( CTIME )

return ( val( substr( CTIME, 7, 2 ) ) / 60 ) + ;
       ( val( substr( CTIME, 4, 2 ) )      ) + ;
       ( val( substr( CTIME, 1, 2 ) ) * 60 )

*!*********************************************************************
*!
*!       Function: TTOM()
*!
*!     Parameters: A Time string in HH:MM:SS format
*!
*!        Returns: The number of minutes since midnight
*!                 seconds are rounded off
*!
*!*********************************************************************
function TTOM( CTIME )

return ( val( substr( CTIME, 4, 2 ) )      ) + ;
       ( val( substr( CTIME, 1, 2 ) ) * 60 )

*!*********************************************************************
*!
*!       Function: MINS2HUND()
*!
*!     Parameters: Minutes in numeric format between 0 and 60 inc.
*!
*!        Returns: Hundredths of hours
*!
*!*********************************************************************
function MINS2HUND( MINUTES )

return MINUTES / 60

*!*****************************************************************************
*!
*!       Function: PRB_INT()
*!
*!*****************************************************************************
function PRB_INT( SOMENUMBER, length, NUM_DECIMALS)

LOCAL NEGATIVE := ( SOMENUMBER < 0 )
LOCAL SOMESTRING, dotat

default NUM_DECIMALS to 0
default length to 19

if NEGATIVE
  SOMENUMBER := abs( SOMENUMBER )
endif

SOMENUMBER += .0000000000000005

SOMESTRING := ALLTRIM( str( SOMENUMBER) )

dotat := at( '.', somestring )

do case
case NUM_DECIMALS == 0
   if dotat > 0
      somestring := left( somestring, dotat - 1 )
   endif
case NUM_DECIMALS > 0
   if dotat > 0
      somestring := left( somestring, dotat + num_decimals )
   endif
endcase

if NEGATIVE
  SOMESTRING := '-' + SOMESTRING
endif

return val( SOMESTRING )

*!*********************************************************************
*!
*!       Function: JUL_PACK()
*!
*!     Parameters: A numeric Julian number
*!
*!        Returns: A packed 5 character Julian
*!
*!*********************************************************************
function JUL_PACK( nJulian )

local cJulian := strtran( strzero( nJulian, 11, 5 ), '.' )
local cPackJul := ''
local x

for x := 1 to 10 step 2
   cPackJul += chr( val( substr( cJulian, x, 2 ) ) )
next

return cPackJul

*!*********************************************************************
*!
*!       Function: JUL_UNPACK()
*!
*!     Parameters: A packed Julian number (5 character)
*!
*!        Returns: A numeric Julian
*!
*!*********************************************************************
function JUL_UNPACK( cPackJul )

local cJulian := ''
local x

for x := 1 to 5
   cJulian += strzero(asc(substr(cPackJul,x,1)),2)
next

return val(left(cJulian,5)+'.'+substr(cJulian,6,5))

